'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _reactServerModuleTagger = require('react-server-module-tagger');

var _reactServerModuleTagger2 = _interopRequireDefault(_reactServerModuleTagger);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function () {
	return {
		visitor: {
			Identifier: function Identifier(p, state) {
				var node = p.node;
				var name = node.name;

				var tokens = getTokens(state);

				if (tokens.has(name)) {
					var trim = state.opts.trim;
					var prefix = state.opts.prefix;
					var parent = _path2.default.resolve(_path2.default.join(process.cwd(), '..')) + _path2.default.sep;
					var filePath = this.file.opts.filename.replace(parent, '');
					var options = JSON.parse((0, _reactServerModuleTagger2.default)({ filePath: filePath, trim: trim, prefix: prefix }));

					p.replaceWith(convertObjectToObjectExpression(options));
				}
			},
			CallExpression: function CallExpression(p, state) {
				var _this = this;

				var node = p.node;
				var callee = node.callee;
				var name = callee.name;

				var tokens = getTokens(state);

				if (tokens.has(name)) {
					(function () {
						var trim = state.opts.trim;
						var prefix = state.opts.prefix;
						var parent = _path2.default.resolve(_path2.default.join(process.cwd(), '..')) + _path2.default.sep;
						var filePath = _this.file.opts.filename.replace(parent, '');
						var opts = {};
						node.arguments[0].properties.forEach(function (prop) {
							opts[prop.key.name] = prop.value.value;
						});
						var options = JSON.parse((0, _reactServerModuleTagger2.default)({ filePath: filePath, trim: trim, prefix: prefix, opts: opts }));

						p.replaceWith(convertObjectToObjectExpression(options));
					})();
				}
			}
		}
	};
};

function convertObjectToObjectExpression(obj) {
	var properties = [];
	Object.keys(obj).forEach(function (key) {
		var literal = void 0;
		switch (_typeof(obj[key])) {
			case 'string':
				literal = t.stringLiteral(obj[key]);
				break;
			case 'number':
				literal = t.numericLiteral(obj[key]);
				break;
			case 'object':
				literal = convertObjectToObjectExpression(obj[key]);
				break;
			default:
				throw new Error('got unexpected objecy property type of ' + _typeof(obj[key]));
		}
		properties.push(t.objectProperty(t.identifier(key), literal));
	});
	return t.objectExpression(properties);
}

function getTokens(state) {
	if (state.opts.tokens) {
		return new Set(state.opts.tokens);
	} else {
		return new Set(["__LOGGER__", "__CHANNEL__", "__CACHE__"]);
	}
}